//
//  PAClock.h
//  PomfortRendering2
//
//  Created by Ulrich Zurucker on 31/10/13.
//  Copyright (c) 2013 Pomfort Software. All rights reserved.
//

#ifndef PomfortRendering2_PAClock_h
#define PomfortRendering2_PAClock_h

#ifdef __cplusplus
extern "C" {
#endif
    
#include <CoreFoundation/CoreFoundation.h>
#include <AudioToolbox/AudioToolbox.h>

/**
 PAClock provides low level calculations from sample time <> fram index domain.
 To do so, PAClock needs to be aware of the sample rate of the time domain
 driving audio device and needs a frame rate. The clock needs a reference to the
 buffered audio provider. The clock will set the the play head position of the 
 audio provider when its frame index is set. The clock ensures sync of audio and
 clock by calling PAClockEnsureSync(); refer to this function for more information.
 All PAClock calls are thread-safe.
 */

/**
 Possible errors
 */
enum {
	kPAClockErr_Alloc				= 'mem!',	// fail on malloc
	kPAClockErr_SampleTimeInvalid	= 'stiv',	// @sa PAClockSetCurrentTimestamp
	kPAClockErr_Invalid				= '!vld'	// other errors
};

typedef struct PAClock_t* PAClock;

/**
 creates a new PAClock instance
 */
extern OSStatus PAClockNew(PAClock* outClock);

/**
 Disposes a PAClock instance
 */
OSStatus	PAClockDispose(PAClock);

/**
 Set the sample rate property of the clock, msut only be called when stopped.
 */
OSStatus	PAClockSetSampleRate(PAClock clock, Float64 nominalSampleRate);

/**
 Set the frame rate property of the clock, must only be called when stopped.
 */
OSStatus	PAClockSetFrameRate(PAClock, double);

/**
 Get the framrate property of the clock
 */
double		PAClockGetFrameRate(const PAClock);

/**
 returns the current frame index, based on the last audio-time stamp passed in
 PAClockSetCurrentTimestamp.
 @param clock the clock object
 @param	outFrameIndex on successful return, the current frame index.
 @return noErr on success.
 */
OSStatus	PAClockGetCurrentFrameIndex(const PAClock clock, SInt64* outFrameIndex);

/**
 Sets the current frame index.
 Used for timeline jumps; sets an internal offset to the clock.
 */
OSStatus	PAClockSetCurrentFrameIndex(const PAClock clock, SInt64 inFrameIndex);

/**
 Proceed the clock a given number of samples
 */
OSStatus    PAClockProceedSamples(PAClock, UInt32);

OSStatus    PAClockGetSamplePosition(const PAClock, SInt64*);
    
OSStatus    PAClockSetSamplePosition(const PAClock, SInt64);

#ifdef __cplusplus
}
#endif

#endif
